@@ -51,9 +51,9 @@ pub use managers::moka::MokaManager;
 pub use moka::future::{Cache as MokaCache, CacheBuilder as MokaCacheBuilder};
 
 // Custom headers used to indicate cache status (hit or miss)
-/// `x-cache` header: Did this proxy serve the result from cache (HIT for yes, MISS for no)
+/// `x-cache` header: Value will be HIT if the response was served from cache, MISS if not
 pub const XCACHE: &str = "x-cache";
-/// `x-cache-lookup` header: Did the proxy have a cacheable response to the request (HIT for yes, MISS for no)
+/// `x-cache-lookup` header: Value will be HIT if a response existed in cache, MISS if not
 pub const XCACHELOOKUP: &str = "x-cache-lookup";
 
 /// Represents a basic cache status
@@ -112,13 +112,13 @@ pub struct HttpResponse {
 }
 
 impl HttpResponse {
-    /// Returns http::response::Parts
+    /// Returns `http::response::Parts`
     pub fn parts(&self) -> Result<response::Parts> {
         let mut converted =
             response::Builder::new().status(self.status).body(())?;
         {
             let headers = converted.headers_mut();
-            for header in self.headers.iter() {
+            for header in &self.headers {
                 headers.insert(
                     http::header::HeaderName::from_str(header.0.as_str())?,
                     http::HeaderValue::from_str(header.1.as_str())?,
@@ -129,25 +129,26 @@ impl HttpResponse {
     }
 
     /// Returns the status code of the warning header if present
+    #[must_use]
     pub fn warning_code(&self) -> Option<usize> {
-        self.headers.get("Warning").and_then(|hdr| {
+        self.headers.get("warning").and_then(|hdr| {
             hdr.as_str().chars().take(3).collect::<String>().parse().ok()
         })
     }
 
     /// Adds a warning header to a response
-    pub fn add_warning(&mut self, url: Url, code: usize, message: &str) {
-        // Warning    = "Warning" ":" 1#warning-value
+    pub fn add_warning(&mut self, url: &Url, code: usize, message: &str) {
+        // warning    = "warning" ":" 1#warning-value
         // warning-value = warn-code SP warn-agent SP warn-text [SP warn-date]
         // warn-code  = 3DIGIT
         // warn-agent = ( host [ ":" port ] ) | pseudonym
         //                 ; the name or pseudonym of the server adding
-        //                 ; the Warning header, for use in debugging
+        //                 ; the warning header, for use in debugging
         // warn-text  = quoted-string
         // warn-date  = <"> HTTP-date <">
         // (https://tools.ietf.org/html/rfc2616#section-14.46)
         self.headers.insert(
-            "Warning".to_string(),
+            "warning".to_string(),
             format!(
                 "{} {} {:?} \"{}\"",
                 code,
@@ -160,11 +161,11 @@ impl HttpResponse {
 
     /// Removes a warning header from a response
     pub fn remove_warning(&mut self) {
-        self.headers.remove("Warning");
+        self.headers.remove("warning");
     }
 
-    /// Update the headers from http::response::Parts
-    pub fn update_headers(&mut self, parts: response::Parts) -> Result<()> {
+    /// Update the headers from `http::response::Parts`
+    pub fn update_headers(&mut self, parts: &response::Parts) -> Result<()> {
         for header in parts.headers.iter() {
             self.headers.insert(
                 header.0.as_str().to_string(),
@@ -175,33 +176,27 @@ impl HttpResponse {
     }
 
     /// Checks if the Cache-Control header contains the must-revalidate directive
+    #[must_use]
     pub fn must_revalidate(&self) -> bool {
-        if let Some(val) = self.headers.get(CACHE_CONTROL.as_str()) {
+        self.headers.get(CACHE_CONTROL.as_str()).map_or(false, |val| {
             val.as_str().to_lowercase().contains("must-revalidate")
-        } else {
-            false
-        }
+        })
     }
 
     /// Adds the custom `x-cache` header to the response
-    pub fn cache_status(&mut self, hit_or_miss: HitOrMiss) -> Result<()> {
+    pub fn cache_status(&mut self, hit_or_miss: HitOrMiss) {
         self.headers.insert(XCACHE.to_string(), hit_or_miss.to_string());
-        Ok(())
     }
 
     /// Adds the custom `x-cache-lookup` header to the response
-    pub fn cache_lookup_status(
-        &mut self,
-        hit_or_miss: HitOrMiss,
-    ) -> Result<()> {
+    pub fn cache_lookup_status(&mut self, hit_or_miss: HitOrMiss) {
         self.headers.insert(XCACHELOOKUP.to_string(), hit_or_miss.to_string());
-        Ok(())
     }
 }
 
 /// A trait providing methods for storing, reading, and removing cache records.
 #[async_trait::async_trait]
-pub trait CacheManager {
+pub trait CacheManager: Send + Sync + 'static {
     /// Attempts to pull a cached response and related policy from cache.
     async fn get(
         &self,
@@ -222,7 +217,7 @@ pub trait CacheManager {
 
 /// Describes the functionality required for interfacing with HTTP client middleware
 #[async_trait::async_trait]
-pub trait Middleware {
+pub trait Middleware: Send {
     /// Determines if the request method is either GET or HEAD
     fn is_method_get_head(&self) -> bool;
     /// Returns a new cache policy with default options
@@ -234,7 +229,7 @@ pub trait Middleware {
         options: CacheOptions,
     ) -> Result<CachePolicy>;
     /// Attempts to update the request headers with the passed `http::request::Parts`
-    fn update_headers(&mut self, parts: request::Parts) -> Result<()>;
+    fn update_headers(&mut self, parts: &request::Parts) -> Result<()>;
     /// Attempts to force the "no-cache" directive on the request
     fn force_no_cache(&mut self) -> Result<()>;
     /// Attempts to construct `http::request::Parts` from the request
@@ -282,11 +277,11 @@ impl TryFrom<http::Version> for HttpVersion {
 
     fn try_from(value: http::Version) -> Result<Self> {
         Ok(match value {
-            http::Version::HTTP_09 => HttpVersion::Http09,
-            http::Version::HTTP_10 => HttpVersion::Http10,
-            http::Version::HTTP_11 => HttpVersion::Http11,
-            http::Version::HTTP_2 => HttpVersion::H2,
-            http::Version::HTTP_3 => HttpVersion::H3,
+            http::Version::HTTP_09 => Self::Http09,
+            http::Version::HTTP_10 => Self::Http10,
+            http::Version::HTTP_11 => Self::Http11,
+            http::Version::HTTP_2 => Self::H2,
+            http::Version::HTTP_3 => Self::H3,
             _ => return Err(CacheError::BadVersion),
         })
     }
@@ -295,11 +290,11 @@ impl TryFrom<http::Version> for HttpVersion {
 impl From<HttpVersion> for http::Version {
     fn from(value: HttpVersion) -> Self {
         match value {
-            HttpVersion::Http09 => http::Version::HTTP_09,
-            HttpVersion::Http10 => http::Version::HTTP_10,
-            HttpVersion::Http11 => http::Version::HTTP_11,
-            HttpVersion::H2 => http::Version::HTTP_2,
-            HttpVersion::H3 => http::Version::HTTP_3,
+            HttpVersion::Http09 => Self::HTTP_09,
+            HttpVersion::Http10 => Self::HTTP_10,
+            HttpVersion::Http11 => Self::HTTP_11,
+            HttpVersion::H2 => Self::HTTP_2,
+            HttpVersion::H3 => Self::HTTP_3,
         }
     }
 }
@@ -310,11 +305,11 @@ impl TryFrom<http_types::Version> for HttpVersion {
 
     fn try_from(value: http_types::Version) -> Result<Self> {
         Ok(match value {
-            http_types::Version::Http0_9 => HttpVersion::Http09,
-            http_types::Version::Http1_0 => HttpVersion::Http10,
-            http_types::Version::Http1_1 => HttpVersion::Http11,
-            http_types::Version::Http2_0 => HttpVersion::H2,
-            http_types::Version::Http3_0 => HttpVersion::H3,
+            http_types::Version::Http0_9 => Self::Http09,
+            http_types::Version::Http1_0 => Self::Http10,
+            http_types::Version::Http1_1 => Self::Http11,
+            http_types::Version::Http2_0 => Self::H2,
+            http_types::Version::Http3_0 => Self::H3,
             _ => return Err(CacheError::BadVersion),
         })
     }
@@ -324,11 +319,11 @@ impl TryFrom<http_types::Version> for HttpVersion {
 impl From<HttpVersion> for http_types::Version {
     fn from(value: HttpVersion) -> Self {
         match value {
-            HttpVersion::Http09 => http_types::Version::Http0_9,
-            HttpVersion::Http10 => http_types::Version::Http1_0,
-            HttpVersion::Http11 => http_types::Version::Http1_1,
-            HttpVersion::H2 => http_types::Version::Http2_0,
-            HttpVersion::H3 => http_types::Version::Http3_0,
+            HttpVersion::Http09 => Self::Http0_9,
+            HttpVersion::Http10 => Self::Http1_0,
+            HttpVersion::Http11 => Self::Http1_1,
+            HttpVersion::H2 => Self::Http2_0,
+            HttpVersion::H3 => Self::Http3_0,
         }
     }
 }
@@ -339,7 +334,7 @@ pub use http_cache_semantics::CacheOptions;
 
 /// Caches requests according to http spec.
 #[derive(Debug, Clone)]
-pub struct HttpCache<T: CacheManager + Send + Sync + 'static> {
+pub struct HttpCache<T: CacheManager> {
     /// Determines the manager behavior.
     pub mode: CacheMode,
     /// Manager instance that implements the [`CacheManager`] trait.
@@ -351,7 +346,7 @@ pub struct HttpCache<T: CacheManager + Send + Sync + 'static> {
 }
 
 #[allow(dead_code)]
-impl<T: CacheManager + Send + Sync + 'static> HttpCache<T> {
+impl<T: CacheManager> HttpCache<T> {
     /// Attempts to run the passed middleware along with the cache
     pub async fn run(
         &self,
@@ -363,27 +358,23 @@ impl<T: CacheManager + Send + Sync + 'static> HttpCache<T> {
         if !is_cacheable {
             return self.remote_fetch(&mut middleware).await;
         }
-        if let Some(store) = self
-            .manager
-            .get(&middleware.method()?.to_uppercase(), &middleware.url()?)
-            .await?
-        {
+        let method = middleware.method()?.to_uppercase();
+        let url = middleware.url()?;
+        if let Some(store) = self.manager.get(&method, &url).await? {
             let (mut res, policy) = store;
-            res.cache_lookup_status(HitOrMiss::HIT)?;
-            let res_url = res.url.clone();
+            res.cache_lookup_status(HitOrMiss::HIT);
             if let Some(warning_code) = res.warning_code() {
                 // https://tools.ietf.org/html/rfc7234#section-4.3.4
                 //
                 // If a stored response is selected for update, the cache MUST:
                 //
-                // * delete any Warning header fields in the stored response with
+                // * delete any warning header fields in the stored response with
                 //   warn-code 1xx (see Section 5.5);
                 //
-                // * retain any Warning header fields in the stored response with
+                // * retain any warning header fields in the stored response with
                 //   warn-code 2xx;
                 //
-                #[allow(clippy::manual_range_contains)]
-                if warning_code >= 100 && warning_code < 200 {
+                if (100..200).contains(&warning_code) {
                     res.remove_warning();
                 }
             }
@@ -395,7 +386,7 @@ impl<T: CacheManager + Send + Sync + 'static> HttpCache<T> {
                 CacheMode::NoCache => {
                     middleware.force_no_cache()?;
                     let mut res = self.remote_fetch(&mut middleware).await?;
-                    res.cache_lookup_status(HitOrMiss::HIT)?;
+                    res.cache_lookup_status(HitOrMiss::HIT);
                     Ok(res)
                 }
                 CacheMode::ForceCache | CacheMode::OnlyIfCached => {
@@ -403,8 +394,12 @@ impl<T: CacheManager + Send + Sync + 'static> HttpCache<T> {
                     // SHOULD be included if the cache is intentionally disconnected from
                     // the rest of the network for a period of time.
                     // (https://tools.ietf.org/html/rfc2616#section-14.46)
-                    res.add_warning(res_url, 112, "Disconnected operation");
-                    res.cache_status(HitOrMiss::HIT)?;
+                    res.add_warning(
+                        &res.url.clone(),
+                        112,
+                        "Disconnected operation",
+                    );
+                    res.cache_status(HitOrMiss::HIT);
                     Ok(res)
                 }
                 _ => self.remote_fetch(&mut middleware).await,
@@ -415,13 +410,13 @@ impl<T: CacheManager + Send + Sync + 'static> HttpCache<T> {
                     // ENOTCACHED
                     let mut res = HttpResponse {
                         body: b"GatewayTimeout".to_vec(),
-                        headers: Default::default(),
+                        headers: HashMap::default(),
                         status: 504,
                         url: middleware.url()?,
                         version: HttpVersion::Http11,
                     };
-                    res.cache_status(HitOrMiss::MISS)?;
-                    res.cache_lookup_status(HitOrMiss::MISS)?;
+                    res.cache_status(HitOrMiss::MISS);
+                    res.cache_lookup_status(HitOrMiss::MISS);
                     Ok(res)
                 }
                 _ => self.remote_fetch(&mut middleware).await,
@@ -434,32 +429,24 @@ impl<T: CacheManager + Send + Sync + 'static> HttpCache<T> {
         middleware: &mut impl Middleware,
     ) -> Result<HttpResponse> {
         let mut res = middleware.remote_fetch().await?;
-        res.cache_status(HitOrMiss::MISS)?;
-        res.cache_lookup_status(HitOrMiss::MISS)?;
+        res.cache_status(HitOrMiss::MISS);
+        res.cache_lookup_status(HitOrMiss::MISS);
         let policy = match self.options {
             Some(options) => middleware.policy_with_options(&res, options)?,
             None => middleware.policy(&res)?,
         };
-        let is_cacheable = middleware.is_method_get_head()
+        let is_get_head = middleware.is_method_get_head();
+        let is_cacheable = is_get_head
             && self.mode != CacheMode::NoStore
             && self.mode != CacheMode::Reload
             && res.status == 200
             && policy.is_storable();
+        let url = middleware.url()?;
+        let method = middleware.method()?.to_uppercase();
         if is_cacheable {
-            Ok(self
-                .manager
-                .put(
-                    &middleware.method()?.to_uppercase(),
-                    &middleware.url()?,
-                    res,
-                    policy,
-                )
-                .await?)
-        } else if !middleware.is_method_get_head() {
-            match self.manager.delete("GET", &middleware.url()?).await {
-                Ok(()) => {}
-                Err(_e) => {}
-            }
+            Ok(self.manager.put(&method, &url, res, policy).await?)
+        } else if !is_get_head {
+            self.manager.delete("GET", &url).await.ok();
             Ok(res)
         } else {
             Ok(res)
@@ -476,18 +463,18 @@ impl<T: CacheManager + Send + Sync + 'static> HttpCache<T> {
             policy.before_request(&middleware.parts()?, SystemTime::now());
         match before_req {
             BeforeRequest::Fresh(parts) => {
-                cached_res.update_headers(parts)?;
-                cached_res.cache_status(HitOrMiss::HIT)?;
-                cached_res.cache_lookup_status(HitOrMiss::HIT)?;
+                cached_res.update_headers(&parts)?;
+                cached_res.cache_status(HitOrMiss::HIT);
+                cached_res.cache_lookup_status(HitOrMiss::HIT);
                 return Ok(cached_res);
             }
             BeforeRequest::Stale { request: parts, matches } => {
                 if matches {
-                    middleware.update_headers(parts)?;
+                    middleware.update_headers(&parts)?;
                 }
             }
         }
-        let req_url = middleware.url()?.clone();
+        let req_url = middleware.url()?;
         match middleware.remote_fetch().await {
             Ok(mut cond_res) => {
                 let status = StatusCode::from_u16(cond_res.status)?;
@@ -498,11 +485,11 @@ impl<T: CacheManager + Send + Sync + 'static> HttpCache<T> {
                     //   due to an inability to reach the server.
                     // (https://tools.ietf.org/html/rfc2616#section-14.46)
                     cached_res.add_warning(
-                        req_url.clone(),
+                        &req_url,
                         111,
                         "Revalidation failed",
                     );
-                    cached_res.cache_status(HitOrMiss::HIT)?;
+                    cached_res.cache_status(HitOrMiss::HIT);
                     Ok(cached_res)
                 } else if cond_res.status == 304 {
                     let after_res = policy.after_response(
@@ -514,19 +501,15 @@ impl<T: CacheManager + Send + Sync + 'static> HttpCache<T> {
                         AfterResponse::Modified(new_policy, parts)
                         | AfterResponse::NotModified(new_policy, parts) => {
                             policy = new_policy;
-                            cached_res.update_headers(parts)?;
+                            cached_res.update_headers(&parts)?;
                         }
                     }
-                    cached_res.cache_status(HitOrMiss::HIT)?;
-                    cached_res.cache_lookup_status(HitOrMiss::HIT)?;
+                    cached_res.cache_status(HitOrMiss::HIT);
+                    cached_res.cache_lookup_status(HitOrMiss::HIT);
+                    let method = middleware.method()?.to_uppercase();
                     let res = self
                         .manager
-                        .put(
-                            &middleware.method()?.to_uppercase(),
-                            &req_url,
-                            cached_res,
-                            policy,
-                        )
+                        .put(&method, &req_url, cached_res, policy)
                         .await?;
                     Ok(res)
                 } else if cond_res.status == 200 {
@@ -535,20 +518,16 @@ impl<T: CacheManager + Send + Sync + 'static> HttpCache<T> {
                             .policy_with_options(&cond_res, options)?,
                         None => middleware.policy(&cond_res)?,
                     };
-                    cond_res.cache_status(HitOrMiss::MISS)?;
-                    cond_res.cache_lookup_status(HitOrMiss::HIT)?;
+                    cond_res.cache_status(HitOrMiss::MISS);
+                    cond_res.cache_lookup_status(HitOrMiss::HIT);
+                    let method = middleware.method()?.to_uppercase();
                     let res = self
                         .manager
-                        .put(
-                            &middleware.method()?.to_uppercase(),
-                            &req_url,
-                            cond_res,
-                            policy,
-                        )
+                        .put(&method, &req_url, cond_res, policy)
                         .await?;
                     Ok(res)
                 } else {
-                    cached_res.cache_status(HitOrMiss::HIT)?;
+                    cached_res.cache_status(HitOrMiss::HIT);
                     Ok(cached_res)
                 }
             }
@@ -561,8 +540,12 @@ impl<T: CacheManager + Send + Sync + 'static> HttpCache<T> {
                     //   because an attempt to revalidate the response failed,
                     //   due to an inability to reach the server.
                     // (https://tools.ietf.org/html/rfc2616#section-14.46)
-                    cached_res.add_warning(req_url, 111, "Revalidation failed");
-                    cached_res.cache_status(HitOrMiss::HIT)?;
+                    cached_res.add_warning(
+                        &req_url,
+                        111,
+                        "Revalidation failed",
+                    );
+                    cached_res.cache_status(HitOrMiss::HIT);
                     Ok(cached_res)
                 }
             }
